/*
** Copyright (c) 2016 CA.  All rights reserved.
** This software may not be duplicated, disclosed or reproduced in whole or
** in part for any purpose except as authorized by the applicable license agreement,
** without the express written authorization of CA. All authorized reproductions
** must be marked with this language.
**
** TO THE EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS
** SOFTWARE “AS IS” WITHOUT WARRANTY OF ANY KIND, INCLUDING
** WITHOUT LIMITATION, ANY IMPLIED WARRANTIES OF MERCHANTABILITY,
** FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.  IN NO EVENT
** WILL CA BE LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS
** OR DAMAGE, DIRECT OR INDIRECT, FROM THE USE OF THIS MATERIAL,
** INCLUDING WITHOUT LIMITATION, LOST PROFITS, BUSINESS
** INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS EXPRESSLY
** ADVISED OF SUCH LOSS OR DAMAGE.
*/
/*======================================================================================
CA Single Sign-On RESTful API - Basic Sample JavaScript Client App
======================================================================================*/
const TB1 = '<table>';
const TB2 = '</table>';
const T1 = '<tr><td>';
const T2 = '</td><td>';
const T3 = '</td></tr>';
const A1 = '<a href="';
const A2 = '" target="_blank">';
const A3 = '</a>';
const I1 = '<input id="';
const I2 = '" type="text" value="';
const I3 = '">';

/**
 * @function getJSON
 * @desc Invokes ajax call for HTTP GET request
*/
function getJSON() {
    var str = TB1;

    //  get hold of the authentication JWT
    getJwtToken();

    //  perform GET only if JWT was received from the Authn Service
    if (Boolean(jwtToken.getValue())) {

        return $.ajax({
            url: $(inputHref).val(),
            type: "GET",
            async: false,
            dataType: "json",
            headers: {
                "Authorization": "Bearer " + jwtToken.getValue()
            },
            success: function (data) { //function to execute upon successful request

                console.log(data);

                jsonObject.setValue(data);
                $('#rawJSON').html("<b>Data from GET request:</b><br>" + JSON.stringify(data));
                $.each(data['data'], function (k, v) {
                    str += T1 + k + T2;
                    if (k.includes('Link')) {
                        str += A1 + v['href'] + A2 + v['path'] + A3 + T3;
                    } else {
                        str += I1 + k + I2 + v + I3 + T3;
                    }
                });
                str += TB2;
                $('#result').html(str);
            },
            error: function (data) {
                console.log('Rest API Error: ' + data.status + ' ' + data.responseText);
            }

        });
    }
}

/**
 * @function putJSON
 * @desc Invokes ajax call for HTTP PUT request (Update operation)
*/
function putJSON() {

    //  get hold of the authentication JWT
    getJwtToken();

    //  perform GET only if JWT was received from the Authn Service
    if (Boolean(jwtToken.getValue())) {

        var obj = jsonObject.getValue();
        obj['data']['Name'] = $(Name).val();
        obj['data']['Desc'] = $(Desc).val();
        obj['data']['RealmHintAttrId'] = parseInt($(RealmHintAttrId).val());

        $('#updateData').html("<b>Data for PUT request:</b><br>" + JSON.stringify(obj['data']));

        return $.ajax({
            url: $(inputHref).val(),
            method: "PUT",
            data: JSON.stringify(obj['data']),
            headers: {
                "Authorization": "Bearer " + jwtToken.getValue()
            },
            success: function (data) { //function to execute upon successful request
            }
        });
    }
}

/**
 * @function deleteJSON
 * @desc Invokes ajax call for HTTP DELETE request
*/
function deleteJSON() {

    //  get hold of the authentication JWT
    getJwtToken();

    //  perform GET only if JWT was received from the Authn Service
    if (Boolean(jwtToken.getValue())) {

        console.log($(inputHref).val());
        $.ajax({
            url: $(inputHref).val(),
            method: "DELETE",
            headers: {
                "Authorization": "Bearer " + jwtToken.getValue()
            },
            success: function (data) {
            }
        });
    }
}

/**
 * @function jsonObject
 * @desc Stores JSON object
*/
var jsonObject = function () {
    var data = {};
    return {
        getValue: function(){
          return data;
        },
        setValue: function (x) {
            data = x;
            return data;
        }
    };
}();


/**
 *  @function getJwtToken get hold of the authentication JWT
 *  @desc Get hold of the authentication JWT.  If it exists, i.e., already fetched due to a prior call, then all set.
 *  Otherwise access the Authentication service and get one.
*/
function getJwtToken() {

    //  check for JWT token
    if (!Boolean(jwtToken.getValue())) {

        //  fetch the token from the authentication service
        return $.ajax({
            url: $(securityHref).val(),
            type: "POST",
            async: false,
            dataType: "json",
            headers: {
                "Authorization": "Basic " + btoa($(userName).val() + ":" + $(pwd).val())
            },
            success: function (data) { //function to execute upon successful request
                console.log(data);
                jwtToken.setValue(data.sessionkey);
            }
        });
    }

}


/**
 * @function jwtToken
 * @desc Stores JWT
*/
var jwtToken = function () {
    var sessionSpec = "";
    return {
        getValue: function () {
            return sessionSpec;
        },
        setValue: function (x) {
            sessionSpec = x;
            return sessionSpec;
        }
    };
}();
