/*
 * Copyright (c) 2016 CA.  All rights reserved.
 *
 * This software and all information contained therein is confidential
 * and proprietary and shall not be duplicated, used, disclosed or
 * disseminated in any way except as authorized by the applicable
 * license agreement, without the express written permission of CA.
 * All authorized reproductions must be marked with this language.
 *
 * EXCEPT AS SET FORTH IN THE APPLICABLE LICENSE AGREEMENT, TO THE
 * EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS SOFTWARE
 * WITHOUT WARRANTY OF ANY KIND, INCLUDING WITHOUT LIMITATION, ANY
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR
 * PURPOSE.  IN NO EVENT WILL CA BE LIABLE TO THE END USER OR ANY
 * THIRD PARTY FOR ANY LOSS OR DAMAGE, DIRECT OR INDIRECT, FROM THE
 * USE OF THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, LOST PROFITS,
 * BUSINESS INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS
 * EXPRESSLY ADVISED OF SUCH LOSS OR DAMAGE.
 *
 */

package sample.restclient;

import static sample.restclient.JSON.gson;
import java.util.HashMap;
import java.util.Map;

/**
 * a Java object representation of the 'data' field of a JSON object
 *
 */
public class ObjectData {

    Map<String, Object> _data;

    @SuppressWarnings("unchecked")
    public <T> T get(String key) {
        if(_data == null){
            return null;
        }
        return (T) _data.get(key);
    }

    public void put(String key, Object val){
        if(_data == null){
            _data = new HashMap<String, Object>();
        }
        _data.put(key, val);
    }

    public String toJson() {
        return gson.toJson(this);
    }

    static public ObjectData fromJson(String data) {
       return gson.fromJson(data, ObjectData.class);
    }

    @Override
    public boolean equals(Object that){
        if(that == null || !(that instanceof ObjectData))
            return false;

        Map<String, Object> dt = ((ObjectData)that)._data;
        if(_data == dt)
            return true;

        if(_data == null)
            return false;

        return _data.equals(dt);
    }

    @Override
    public int hashCode(){
        return (_data == null) ? 0 : _data.hashCode();
    }
}
